% Introduction to Seismology
% Earthquake Locations
% Renate Hartog - October 2001
%
% Script to solve a least squares location problem.
%
% I. Input: station latitudes and longitudes and observed P arrival times
% The file lab1.data needs to contain the following columns:
% stla stlo hrP mnP secP hrS mnS secS
% where
%     stla = geographic latitude of station
%     stlo = longitude of station
%      hrP = hour of arrival time of P  (0-24, only integers)
%      mnP = minutes of arrival time of P (0-60, only integers)
%     secP = seconds of arrival time of P (0-60, real) 
%
clear all; clc

deg2rad = pi/180.0;
rad2deg = 180.0/pi;

% Read in travel time curves for P 
phase = 'P';
[Pdist,Pdepth,Pgrid] = read_tt_table(phase);

itermax = input('Maximum number of iterations ? ');

Tshift = zeros(itermax+1,1);
evla = zeros(itermax+1,1);
evlo = zeros(itermax+1,1);
h = zeros(itermax+1,1);

% Get user input
disp('        ')

answer = input('Solve for depth? [y or n] ','s');
if answer == 'y'
   solve_for_depth = 'T';
   nmodel = 4;
else
   solve_for_depth = 'F';
   nmodel = 3;
end
if solve_for_depth == 'F'
   h(1) = input('Give fixed depth of earthquake [km]: ');
   h = h(1)*ones(itermax,1);
else
   h = zeros(itermax,1);
end
answer = input('Use location of earliest arrival as initial event location? [y or n] ','s');
if answer == 'y'
   use_first = 'T';
else
   use_first = 'F';
end
if use_first == 'F'
   evla(1) = input('Give trial latitude in degrees N  : ');
   evlo(1) = input('Give trial longitude in degrees E : ');
end
datafile = input('Give name of file that contains P wave arrival data : ','s');
   
% The datafile contains as many rows as there are stations (ndata)
% Read in 5 columns, let read() decide how many rows there are (-1).

InputMatrix = load(datafile);
stla = InputMatrix(:,1);
stlo = InputMatrix(:,2);
hrP = InputMatrix(:,3);
mnP = InputMatrix(:,4);
secP = InputMatrix(:,5);

szmatrix = size(InputMatrix);
ndata = szmatrix(1);

% II. Determine the location of the station with the shortest P time, 
% call this location the initial event location, the shortest arrival time is To, 
% set the initial event depth to 0 km.

% Determine what the observed arrival time is in seconds
% (note: beware, this assumes that the arrival times do not cross over to the next
% day).

Pmin = 360000;
for i = 1:ndata
   Ptime(i,1) = hrP(i,1)*3600 + mnP(i,1)*60 + secP(i,1);
   if Ptime(i,1) < Pmin
      Pmin = Ptime(i,1);
      imin = i;
      if use_first == 'T'
         evla(1) = stla(i,1);
         evlo(1) = stlo(i,1);
      end
   end
end

% Remove the initial guess of the origin time from the arrival times
% to obtain the 'observed' travel time.

Tp = Ptime - Pmin;
hr0 = floor(Pmin/3600);
rest = Pmin/3600. - hr0;
mn0 = floor(rest*60);
rest2 = rest*60 - mn0;
sec0 = rest2*60;
T0 = [hr0 mn0 sec0];

% Improve the estimate of the origin time, by subtracting the average 'travel time' residual.

X0 = [evla(1), evlo(1), h(1)];

disp(' ');
disp(['Initial estimate of origin time : ' num2str(T0,'%3.0f %3.0f %5.2f')]);
disp(['Initial estimate of location    : ' num2str(X0,'%7.2f deg %7.2f deg %7.2f km')]);
disp(' ');

if solve_for_depth == 'T'
  x = [0.0, h(1), evla(1), evlo(1)]';
else
  x = [0.0, evla(1), evlo(1)]';
end

% III. Calculate epicentral distance, travel time differences and spatial derivatives for
% this estimate. Plot the squared travel time differences and indicate the RMS value.
% (Root mean squared travel time residual = sqrt(mean of squared travel time
% residuals).

iteration = 1;
criterium = 10000.0;
prev_criterium = 1000000.0;
Tshift(1) = 0.;
ttP = 0; 
raypP = 0; 
dTdhP = 0; 

while (iteration < itermax  && criterium > 0.01* 2 * ndata && criterium < prev_criterium) 
 
disp(['Iteration number: ', num2str(iteration,'%3i')]);

%update derivatives and update travel time residuals

%each time, remove the estimate of the origin time shift, 
%to be able to work with travel time residuals.

  for i = 1:ndata ;
     sla = stla(i,1);
     slo = stlo(i,1);
     ela = evla(iteration);
     elo = evlo(iteration);
     depth = h(iteration);

     [delta, az, baz] = distaz(ela,elo,sla,slo);
     azi(i,1) = az;
     [ttP,dTdhP,raypP] = ttime(delta,depth,Pdist,Pdepth,Pgrid);

     resP(i,1) = Tp(i,1) - (Tshift(iteration) + ttP) ; % observed arrival time - predicted arrival time

     % convert angles from degrees to radians
     dTdlat = -1 .* raypP .* cos(deg2rad*az);
     dTdlon = -1 .* raypP .* sin(deg2rad*az) .* cos(deg2rad*ela);
     dTdh = dTdhP;
     dTd0 = 1.; 
     if solve_for_depth == 'T'
       A(i,1) = dTd0; A(i,2) = dTdh; A(i,3) = dTdlat; A(i,4) =dTdlon;
     else
       A(i,1) = dTd0; A(i,2) = dTdlat; A(i,3) =dTdlon;
     end
  end
  
% IV. Solve and update the location estimate with new found values
  dT = resP;

% solve the equation Adx = dT
  dx = inv( A' * A ) * A' * dT;

% update the estimate of the model vector
  x = x + dx;

% calculate the variance and RMS value;
  model_length(iteration) = dx'*dx;
  eps = dT;
  var(iteration) = ( eps'*eps ) ./ (ndata - nmodel);
  RMS(iteration) = sqrt( var(iteration) );
  criterium = RMS(iteration);
  if iteration == 1
     var0 = var(1);
     RMS0 = RMS(1);
  end
  if iteration >= 2
     prev_criterium = RMS(iteration-1);
  end
  disp(['Root Mean Square residual: ', num2str(RMS(iteration),'%7.2f')]);

% we have the next estimate of the location, increase number of interations by 1.
iteration = iteration + 1;

if solve_for_depth == 'T'
  Tshift(iteration) = x(1,1);
  h(iteration) = x(2,1);
  evla(iteration) = x(3,1);
  evlo(iteration) = x(4,1);
else 
  Tshift(iteration) = x(1,1);
  evla(iteration) = x(2,1);
  evlo(iteration) = x(3,1);
end

  disp(['Updated location: h(' num2str(iteration-1) ')= ', num2str(h(iteration),'%7.2f')]);
  disp(['                  evla(' num2str(iteration-1) ')= ' num2str(evla(iteration),'%7.2f')]);
  disp(['                  evlo(' num2str(iteration-1) ')= ' num2str(evlo(iteration),'%7.2f')]);

if (iteration >= itermax)
   reason = 'Maximum number of iterations reached';
elseif (criterium < 0.01* 2 * ndata)
   reason = 'Found optimal solution because RMS value is below 0.01 s' ;
elseif criterium > prev_criterium
   reason = 'Found optimal solution because RMS increased' ;
end

end

% V. repeat steps III and IV until the RMS value is below a certain cut-off (0.1s).
% or until the variance starts to increase again.
% VI. Keep track of the number of iterations. Output the variance reduction, 
% the final RMS value, the best location and the
% 'observed' and calculated travel times.

niter = iteration - 1;
var_red = 100*(var0 - var)/var0;

[RMSmin,imin] = min(RMS);

solution = [h(imin), evla(imin), evlo(imin) ];
Torig = Pmin + Tshift(imin);
hours = fix(Torig/3600.0);
rest = Torig/3600.0 - hours;
minutes = fix(rest * 60.0);
seconds = (rest * 60.0 - minutes) * 60.;
origintime = [hours, minutes, seconds];

% OUTPUT TO MATLAB WINDOW
disp('************************************************************');
disp('                     ')
disp(reason);
disp('                     ')
if solve_for_depth == 'T'
   disp(['Earthquake depth was solved for and determined to be ' num2str(h(imin),'%6.0f') ' km'])
else
   disp(['Earthquake depth was held fixed at ' num2str(h(1,1),'%6.0f') ' km'])
end

if use_first == 'T'
   disp(['Initial location was location of station with first arrival: ' num2str(X0,'%7.2f N %7.2f E %3.0f km')])
else
   disp(['Initial location was: ' num2str(X0,'%7.2f N %7.2f E %6.0f km')])
end
disp(['Initial estimate of origin time was: ' num2str(T0,'%3.0f %3.0f %5.2f')]);
disp('                     ')
disp(['Final solution after ' num2str(niter,'%5.0f') ' iterations'])
disp(['Origin Time: ' num2str([fix(hours) fix(minutes) seconds],'%2.0f %2.0f %5.2f')]);
disp(num2str(solution,'Depth: %7.2f  Lat: %7.2f Lon: %7.2f'));
disp(['Average RMS travel time residual is: ' num2str(RMS(imin),'%7.3f') ' s'])
disp(' ')
disp('Obs P(s)   Pred P(s)   Diff(s)');
for i = 1:ndata ;
  sla = stla(i,1);
  slo = stlo(i,1);
  ela = evla(imin);
  elo = evlo(imin);
  depth = h(imin);

  [delta, az, baz] = distaz(ela,elo,sla,slo);
  azi(i,1) = az;
  [ttP,dTdhP,raypP] = ttime(delta,depth,Pdist,Pdepth,Pgrid);

  resP(i,1) = Tp(i,1) - Tshift(imin) - ttP;
  disp([num2str([(Tp(i,1)-Tshift(imin)),ttP,resP(i,1)],'%10.2f %10.2f %10.2f')]);
  
  % convert angles from degrees to radians
  dTdlat = -1 .* raypP .* cos(deg2rad*az);
  dTdlon = -1 .* raypP .* sin(deg2rad*az) .* cos(deg2rad*ela);
  dTdh = dTdhP;
  dTd0 = 1.;
  if solve_for_depth == 'T'
    A(i,1) = dTd0; A(i,2) = dTdh; A(i,3) = dTdlat; A(i,4) =dTdlon;
  else
    A(i,1) = dTd0; A(i,2) = dTdlat; A(i,3) =dTdlon;
  end
end
disp('************************************************************');

%PLOTTING
%figure(1); clf; hold on
%plot(stlo,stla,'Vb','markersize',8)
%text(stlo+2,stla,num2str((1:length(stlo))'),'color','k')
%plot(evlo(imin),evla(imin),'pr','markersize',10)
%box on; grid on
%xlabel('Longitude (degrees)'); ylabel('Latitude (degrees)')
%hold off

%figure(2); clf
%subplot(2,1,1); hold on
%plot(azi,resP,'o')
%text(azi+2,resP,num2str((1:length(azi))'),'color','r','fontweight','bold')
%title('P arrival time residuals for the final solution')
%xlabel('azimuth to station (deg)'); ylabel('residual (s)')
%grid on; box on; hold off
%subplot(2,1,2);
%plot(1:niter, RMS,'-o')
%title('RMS value as a function of iteration number')
%xlabel('iteration number'); ylabel(' RMS (s)')
%grid on; box on
