function [T,dTdh,dTdD] = ttime(x,h,dist,depths,tgrid)

% Routine to calculate travel times T, dTdh, and dTdD
% by bicubic spline interpolation of travel time curves that
% are read from pre-existing tables. Only P tables and S tables can be read.
% To obtain both dTdh and dTdD I followed the approach of W. Nagy (locsat).
%
% Input: x = epicentral distance in degrees
%        h = earthquake depth in km
%    phase = 'P' or 'S'
% Output:
%        T = the calculated travel time for (x,h) in s
%     dTdh = the estimate of dTdh in s/km
%          = derivative w.r.t. source depth = vertical slowness
%     dTdD = the estimate of dTdD  in s/deg
%          = derivative w.r.t. epicentral distance = ray parameter


% distance- and depth samples, and travel time values at those
% points are already read in by the main script.

szd = size(dist);
nx = szd(1)*szd(2);
szd = size(depths);
nz = szd(1)*szd(2);

% initialize output variables
T = zeros(1,1);
dTdD = zeros(1,1);
dTdh = zeros(1,1);

% determine the subset of the travel time curve to interpolate with splines.

nxmax = 7;
nzmax = 7;

% [jleft] = call('locate',dist,1,'r',nx,2,'i',x,3,'r','out',[1,1],4,'i');
% [ileft] = call('locate',depths,1,'r',nz,2,'i',h,3,'r','out',[1,1],4,'i');

[jleft] = locate(dist,nx,x);
[ileft] = locate(depths,nz,h);

imin = ileft - 3;
imax = ileft + 3;
if (imin) <= 0 
   imin = 1;
   imax = nzmax;
end
if (imax) > nz 
   imax = nz;
   imin = nz - nzmax;
end
jmin = jleft - 3;
jmax = jleft + 3;
if (jmin) <= 0 
   jmin = 1;
   jmax = nxmax;
end
if (jmax) > nx 
   jmax = nx;
   jmin = nx - nxmax;
end
d = dist(jmin:jmax);
t = tgrid(imin:imax,jmin:jmax);
z = depths(imin:imax);
nd = (jmax - jmin) + 1;
nh = (imax - imin) + 1;
dummy = 2e30;

% First interpolate the rows of the t-matrix and create a vector with 2nd
% derivatives:

% [y2atrans] = call('splie2',d,1,'r',z,2,'r',t',3,'r',nd,4,'i',nh,5,'i','out',[nd,nh],6,'r');
[y2atrans] = splie2(d,z,t',nd,nh);

% run mysplin2 to find T and dTdD
%[value,dTdD,dT2dD] = call('mysplin2',d,1,'r',z,2,'r',t',3,'r',y2atrans,4,'r',nn,5,'i',nn,6,'i',x,7,'r',h,8,'r','out',[1,1],9,'r',[1,1],10,'r',[1,1],11,'r') 

[value,dTdD,dT2dD] = mysplin2(d,z,t',y2atrans,nd,nh,x,h);

% Repeat with the transpose
%[y2a] = call('splie2',z,1,'r',d,2,'r',t,3,'r',nn,4,'i',nn,5,'i','out',[nn,nn],6,'r');
[y2a] = splie2(z,d,t,nh,nd);

% Then calculate the value of T and dTdh (and dT2dh)
%[value,dTdh,dT2dh] = call('mysplin2',z,1,'r',d,2,'r',t,3,'r',y2a,4,'r',nn,5,'i',nn,6,'i',h,7,'r',x,8,'r','out',[1,1],9,'r',[1,1],10,'r',[1,1],11,'r')

[value,dTdh,dT2dh] = mysplin2(z,d,t,y2a,nh,nd,h,x);

T = value;